"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
var InternalProvider_1 = require("./InternalProvider");
var ILifetime_1 = require("./ILifetime");
var DependencyError_1 = require("./DependencyError");
var NameSelector_1 = require("./NameSelector");
var Container = /** @class */ (function () {
    function Container(ProviderTemplate) {
        this.dependencyLookup = {};
        this.providerLookup = {};
        this.template = new ProviderTemplate();
        this.intendedDependencies = Object.keys(this.template).reduce(function (a, b) {
            a[b.toLowerCase()] = b;
            return a;
        }, {});
    }
    Container.prototype.addSingleton = function (Dependency, options) {
        if (options === void 0) { options = {}; }
        this.add(ILifetime_1.Singleton, Dependency, options);
    };
    Container.prototype.tryAddSingleton = function (Dependency, options) {
        if (options === void 0) { options = {}; }
        this.tryAdd(ILifetime_1.Singleton, Dependency, options);
    };
    Container.prototype.addTransient = function (Dependency, options) {
        if (options === void 0) { options = {}; }
        this.add(ILifetime_1.Transient, Dependency, options);
    };
    Container.prototype.tryAddTransient = function (Dependency, options) {
        if (options === void 0) { options = {}; }
        this.tryAdd(ILifetime_1.Transient, Dependency, options);
    };
    Container.prototype.add = function (Lifetime, Dependency, options) {
        if (options === void 0) { options = {}; }
        if (!this.tryAdd(Lifetime, Dependency, options)) {
            var providerName = options.name
                ? options.name((0, NameSelector_1.nameSelector)(this.template))
                : this.intendedDependencies[Dependency.name.toLowerCase()];
            throw new DependencyError_1.DependencyError({
                type: DependencyError_1.DependencyErrorType.Duplicate,
                lifetime: providerName,
            });
        }
    };
    Container.prototype.tryAdd = function (Lifetime, Dependency, _a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.factory, factory = _c === void 0 ? function (provider) { return new Dependency(provider); } : _c, name = _b.name;
        var providerName = name
            ? name((0, NameSelector_1.nameSelector)(this.template))
            : this.intendedDependencies[Dependency.name.toLowerCase()];
        if (!providerName)
            throw new DependencyError_1.DependencyError({ type: DependencyError_1.DependencyErrorType.Unknown, lifetime: Dependency });
        if (this.providerLookup[providerName])
            return false;
        if (this.dependencyLookup[Dependency.name])
            return false;
        this.dependencyLookup[Dependency.name] = new Lifetime(this, Dependency.name, providerName, factory);
        this.providerLookup[providerName] = new Lifetime(this, Dependency.name, providerName, factory);
        return true;
    };
    /**
     * Calling this there is no guarantee that it can provide an instance
     */
    Container.prototype.get = function (item) {
        if (typeof item === 'string')
            return this.providerLookup[item];
        else
            return this.dependencyLookup[item.name];
    };
    Container.prototype.preBuildValidate = function () {
        var _this = this;
        var missing = Object.keys(this.template)
            .filter(function (key) { return !_this.providerLookup[key]; })
            .map(function (key) { return new DependencyError_1.DependencyError({ type: DependencyError_1.DependencyErrorType.Existence, lifetime: key }); });
        if (missing.length === 1)
            throw missing[0];
        else if (missing.length > 1)
            throw new DependencyError_1.DependencyMultiError(DependencyError_1.DependencyMultiErrorType.Build, missing);
    };
    Container.prototype.build = function () {
        var _this = this;
        if (this.provider)
            return this.provider;
        var provider = new InternalProvider_1.InternalProvider(this);
        this.preBuildValidate();
        Object.keys(this.template).forEach(function (providerName) {
            var _a;
            var className = (_a = _this.providerLookup[providerName]) === null || _a === void 0 ? void 0 : _a.className;
            Object.defineProperty(provider, providerName, { get: function () { return provider.get(providerName); } });
            if (className !== providerName)
                Object.defineProperty(provider, className, { get: function () { return provider.get(providerName); } });
        });
        return (this.provider = provider);
    };
    return Container;
}());
exports.Container = Container;
